package com.qmth.exam.reserve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.exam.reserve.bean.RichTextBean;
import com.qmth.exam.reserve.bean.apply.ApplyTimePeriodResult;
import com.qmth.exam.reserve.bean.apply.ApplyVO;
import com.qmth.exam.reserve.bean.apply.TicketInfo;
import com.qmth.exam.reserve.dao.StudentApplyDao;
import com.qmth.exam.reserve.entity.StudentApplyEntity;
import com.qmth.exam.reserve.service.ExamReserveService;
import com.qmth.exam.reserve.service.StudentApplyService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class ExamReserveServiceImpl implements ExamReserveService {

    private static final Logger log = LoggerFactory.getLogger(ExamReserveServiceImpl.class);

    @Autowired
    private StudentApplyService studentApplyService;

    @Override
    @Transactional
    public void saveStudentApply(Long studentId, Long examSiteId, Long timePeriodId) {
        LambdaQueryWrapper<StudentApplyEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StudentApplyEntity::getStudentId, studentId);
        wrapper.eq(StudentApplyEntity::getExamSiteId, examSiteId);
        wrapper.eq(StudentApplyEntity::getTimePeriodId, timePeriodId);
        StudentApplyEntity existEntity = studentApplyService.getOne(wrapper);
        if (existEntity != null) {
            if (existEntity.getCancel()) {
                existEntity.setCancel(false);
                existEntity.setOperateId(studentId);
                existEntity.setUpdateTime(System.currentTimeMillis());
                studentApplyService.updateById(existEntity);
            }
            return;
        }

        // todo
        // 考前多少天，禁止考生自主预约（考前是指待预约时段的开始时间）
        // 系统自动预约“任务执行期间”不允许预约

        StudentApplyEntity entity = new StudentApplyEntity();
        entity.setStudentId(studentId);
        entity.setExamSiteId(examSiteId);
        entity.setTimePeriodId(timePeriodId);
        entity.setOperateId(studentId);
        entity.setCancel(false);
        studentApplyService.save(entity);
    }

    @Override
    @Transactional
    public void cancelStudentApply(Long studentId, Long applyId) {
        LambdaUpdateWrapper<StudentApplyEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(StudentApplyEntity::getCancel, true);
        updateWrapper.set(StudentApplyEntity::getOperateId, studentId);
        updateWrapper.set(StudentApplyEntity::getUpdateTime, System.currentTimeMillis());
        updateWrapper.eq(StudentApplyEntity::getId, applyId);
        studentApplyService.update(updateWrapper);
        // todo
        // 考前多少天，禁止考生自主取消预约（考前是指已预约时段的开始时间）
        // 系统自动预约“任务执行期间”不允许取消预约

    }

    @Override
    public List<ApplyVO> getStudentApplyList(Long studentId, Boolean cancel) {
        StudentApplyDao baseMapper = (StudentApplyDao) studentApplyService.getBaseMapper();
        List<ApplyVO> list = baseMapper.getStudentApplyList(studentId, cancel);

        for (ApplyVO vo : list) {
            vo.setShowTicket(false);
            vo.setAllowCancel(true);
            if (StringUtils.isNotEmpty(vo.getTicketNumber())) {
                vo.setShowTicket(true);
            }
            // todo
        }

        return list;
    }

    @Override
    public List<ApplyVO> getStudentApplyListForCurrent(Long studentId) {
        List<ApplyVO> list = this.getStudentApplyList(studentId, null);
        // todo
        return list;
    }

    @Override
    public TicketInfo getApplyTicket(Long studentId, Long applyId) {
        StudentApplyDao baseMapper = (StudentApplyDao) studentApplyService.getBaseMapper();
        TicketInfo info = baseMapper.getStudentApplyTicket(applyId);
        if (info == null || info.getCancel()) {
            throw new StatusException("准考证信息不存在");
        }
        if (!info.getStudentId().equals(studentId)) {
            throw new StatusException("准考证信息不匹配");
        }
        if (StringUtils.isEmpty(info.getTicketNumber())) {
            throw new StatusException("准考证信息尚未生成");
        }
        return info;
    }

    @Override
    public List<String> getApplyDateList(Long studentId) {
        return null;
    }

    @Override
    public ApplyTimePeriodResult getApplyTimePeriodList(Long studentId, Long examSiteId, String date) {
        return null;
    }

    @Override
    public RichTextBean getExamNotice(Long studentId, Long applyTaskId) {
        return null;
    }

    @Override
    public RichTextBean getExamRoomGuide(Long studentId, Long examRoomId) {
        return null;
    }

}
